@@ -7,7 +7,6 @@ use anyhow::Result;
 use ast_grep_core::{NodeMatch as SgNodeMatch, StrDoc};
 use codespan_reporting::files::SimpleFile;
 use std::io::{Stdout, Write};
-use std::sync::Mutex;
 
 type NodeMatch<'a, L> = SgNodeMatch<'a, StrDoc<L>>;
 
@@ -28,15 +27,13 @@ pub enum Platform {
   GitHub,
 }
 
-pub struct CloudPrinter<W: Write + Send + Sync> {
-  writer: Mutex<W>,
+pub struct CloudPrinter<W: Write> {
+  writer: W,
 }
 
-impl<W: Write + Send + Sync> CloudPrinter<W> {
-  pub fn new(w: W) -> Self {
-    Self {
-      writer: Mutex::new(w),
-    }
+impl<W: Write> CloudPrinter<W> {
+  pub fn new(writer: W) -> Self {
+    Self { writer }
   }
 }
 
@@ -46,9 +43,9 @@ impl CloudPrinter<Stdout> {
   }
 }
 
-impl<W: Write + Send + Sync> Printer for CloudPrinter<W> {
+impl<W: Write> Printer for CloudPrinter<W> {
   fn print_rule<'a>(
-    &self,
+    &mut self,
     matches: Matches!('a),
     file: SimpleFile<Cow<str>, &String>,
     rule: &RuleConfig<SgLang>,
@@ -57,16 +54,16 @@ impl<W: Write + Send + Sync> Printer for CloudPrinter<W> {
     print_rule(self, matches, &path, rule)
   }
 
-  fn print_matches<'a>(&self, _m: Matches!('a), _p: &Path) -> Result<()> {
+  fn print_matches<'a>(&mut self, _m: Matches!('a), _p: &Path) -> Result<()> {
     unreachable!()
   }
 
-  fn print_diffs<'a>(&self, _d: Diffs!('a), _p: &Path) -> Result<()> {
+  fn print_diffs<'a>(&mut self, _d: Diffs!('a), _p: &Path) -> Result<()> {
     unreachable!()
   }
 
   fn print_rule_diffs(
-    &self,
+    &mut self,
     diffs: Vec<(Diff<'_>, &RuleConfig<SgLang>)>,
     path: &Path,
   ) -> Result<()> {
@@ -77,13 +74,13 @@ impl<W: Write + Send + Sync> Printer for CloudPrinter<W> {
   }
 }
 
-fn print_rule<'a, W: Write + Send + Sync>(
-  p: &CloudPrinter<W>,
+fn print_rule<'a, W: Write>(
+  p: &mut CloudPrinter<W>,
   matches: Matches!('a),
   path: &Path,
   rule: &RuleConfig<SgLang>,
 ) -> Result<()> {
-  let mut writer = p.writer.lock().expect("should work");
+  let writer = &mut p.writer;
   let level = match rule.severity {
     Severity::Error => "error",
     Severity::Warning => "warning",
@@ -98,7 +95,7 @@ fn print_rule<'a, W: Write + Send + Sync>(
     let end_line = m.end_pos().line() + 1;
     let message = rule.get_message(&m);
     writeln!(
-      &mut writer,
+      writer,
       "::{level} file={name},line={line},endLine={end_line},title={title}::{message}"
     )?;
   }
@@ -115,8 +112,8 @@ mod test {
   fn make_test_printer() -> CloudPrinter<Buffer> {
     CloudPrinter::new(Buffer::no_color())
   }
-  fn get_text(printer: &CloudPrinter<Buffer>) -> String {
-    let buffer = printer.writer.lock().expect("should work");
+  fn get_text(printer: &mut CloudPrinter<Buffer>) -> String {
+    let buffer = &mut printer.writer;
     let bytes = buffer.as_slice();
     std::str::from_utf8(bytes)
       .expect("buffer should be valid utf8")
@@ -142,13 +139,13 @@ language: TypeScript
 
   fn test_output(src: &str, rule_str: &str, expect: &str) {
     let src = src.to_owned();
-    let printer = make_test_printer();
+    let mut printer = make_test_printer();
     let grep = SgLang::from(SupportLang::Tsx).ast_grep(&src);
     let rule = make_rule(rule_str);
     let matches = grep.root().find_all(&rule.matcher);
     let file = SimpleFile::new(Cow::Borrowed("test.tsx"), &src);
     printer.print_rule(matches, file, &rule).unwrap();
-    let actual = get_text(&printer);
+    let actual = get_text(&mut printer);
     assert_eq!(actual, expect);
   }
 
