@@ -2,10 +2,10 @@ use std::path::{Path, PathBuf};
 
 use anyhow::{Context, Result};
 use ast_grep_config::{
-  from_yaml_string, CombinedScan, PreScan, RuleCollection, RuleConfig, SerializableRule,
-  SerializableRuleConfig, SerializableRuleCore, Severity,
+  from_yaml_string, CombinedScan, PreScan, RuleCollection, RuleConfig, Severity,
 };
 use ast_grep_core::{NodeMatch, StrDoc};
+use ast_grep_language::SupportLang;
 use clap::Args;
 use ignore::WalkParallel;
 
@@ -65,6 +65,13 @@ pub struct ScanArg {
   context: ContextArgs,
 }
 
+impl ScanArg {
+  // whether the scan includes all rules available in the project
+  fn include_all_rules(&self) -> bool {
+    self.overwrite.include_all_rules() && self.rule.is_none() && self.inline_rules.is_none()
+  }
+}
+
 pub fn run_with_config(arg: ScanArg, project: Result<ProjectConfig>) -> Result<()> {
   let context = arg.context.get();
   if let Some(_format) = &arg.format {
@@ -94,26 +101,25 @@ fn run_scan<P: Printer + 'static>(
   project: Result<ProjectConfig>,
 ) -> Result<()> {
   if arg.input.stdin {
-    let worker = ScanWithRule::try_new(arg, printer)?;
+    let worker = ScanWithRule::try_new(arg)?;
     // TODO: report a soft error if rules have different languages
-    worker.run_std_in()
+    worker.run_std_in(printer)
   } else {
-    let worker = ScanWithConfig::try_new(arg, printer, project)?;
-    worker.run_path()
+    let worker = ScanWithConfig::try_new(arg, project)?;
+    worker.run_path(printer)
   }
 }
 
-struct ScanWithConfig<Printer> {
+struct ScanWithConfig {
   arg: ScanArg,
-  printer: Printer,
   configs: RuleCollection<SgLang>,
   unused_suppression_rule: RuleConfig<SgLang>,
   trace: ScanTrace,
 }
-impl<P: Printer> ScanWithConfig<P> {
-  fn try_new(arg: ScanArg, printer: P, project: Result<ProjectConfig>) -> Result<Self> {
+impl ScanWithConfig {
+  fn try_new(arg: ScanArg, project: Result<ProjectConfig>) -> Result<Self> {
     let overwrite = RuleOverwrite::new(&arg.overwrite)?;
-    let unused_suppression_rule = unused_suppression_rule_config(&overwrite);
+    let unused_suppression_rule = unused_suppression_rule_config(&arg, &overwrite);
     let (configs, rule_trace) = if let Some(path) = &arg.rule {
       let rules = read_rule_file(path, None)?;
       with_rule_stats(rules)?
@@ -130,53 +136,38 @@ impl<P: Printer> ScanWithConfig<P> {
     trace.print_rules(&configs)?;
     Ok(Self {
       arg,
-      printer,
       configs,
       unused_suppression_rule,
       trace,
     })
   }
 }
-impl<P: Printer> Worker for ScanWithConfig<P> {
+impl Worker for ScanWithConfig {
   type Item = (PathBuf, AstGrep, PreScan);
-  fn consume_items(&self, items: Items<Self::Item>) -> Result<()> {
-    self.printer.before_print()?;
+  fn consume_items<P: Printer>(&self, items: Items<Self::Item>, mut printer: P) -> Result<()> {
+    printer.before_print()?;
     let mut error_count = 0usize;
     for (path, grep, pre_scan) in items {
       let file_content = grep.source().to_string();
       let path = &path;
       let rules = self.configs.get_rule_from_lang(path, *grep.lang());
-      let combined = CombinedScan::new(rules);
+      let mut combined = CombinedScan::new(rules);
+      combined.set_unused_suppression_rule(&self.unused_suppression_rule);
       let interactive = self.arg.output.needs_interactive();
       // exclude_fix rule because we already have diff inspection before
       let scanned = combined.scan(&grep, pre_scan, /* separate_fix*/ interactive);
       if interactive {
-        let diffs = scanned
-          .diffs
-          .into_iter()
-          .map(|(idx, nm)| {
-            let rule = combined.get_rule(idx);
-            (nm, rule)
-          })
-          .collect();
-        match_rule_diff_on_file(path, diffs, &self.printer)?;
+        let diffs = scanned.diffs;
+        match_rule_diff_on_file(path, diffs, &mut printer)?;
       }
-      for (idx, matches) in scanned.matches {
-        let rule = combined.get_rule(idx);
+      for (rule, matches) in scanned.matches {
         if matches!(rule.severity, Severity::Error) {
           error_count = error_count.saturating_add(matches.len());
         }
-        match_rule_on_file(path, matches, rule, &file_content, &self.printer)?;
+        match_rule_on_file(path, matches, rule, &file_content, &mut printer)?;
       }
-      error_count += print_unused_suppressions(
-        path,
-        scanned.unused_suppressions,
-        &self.unused_suppression_rule,
-        &file_content,
-        &self.printer,
-      )?;
     }
-    self.printer.after_print()?;
+    printer.after_print()?;
     self.trace.print()?;
     if error_count > 0 {
       Err(anyhow::anyhow!(EC::DiagnosticError(error_count)))
@@ -186,53 +177,26 @@ impl<P: Printer> Worker for ScanWithConfig<P> {
   }
 }
 
-fn unused_suppression_rule_config(overwrite: &RuleOverwrite) -> RuleConfig<SgLang> {
-  let rule: SerializableRule = serde_json::from_str(r#"{"pattern": "a"}"#).unwrap();
-  let core = SerializableRuleCore {
-    rule,
-    constraints: None,
-    fix: None,
-    transform: None,
-    utils: None,
-  };
+// we should only suggest unused suppression if scan includes all rules
+// otherwise, keep silent about unused suppressions because they may used by other rules
+// this is a "smart" heuristic but user always can override it
+fn default_unused_suppression_rule_severity(arg: &ScanArg) -> Severity {
+  if arg.include_all_rules() {
+    Severity::Hint
+  } else {
+    Severity::Off
+  }
+}
+
+fn unused_suppression_rule_config(arg: &ScanArg, overwrite: &RuleOverwrite) -> RuleConfig<SgLang> {
   let severity = overwrite
     .find("unused-suppression")
     .severity
-    .unwrap_or(Severity::Hint);
-  let config = SerializableRuleConfig::<SgLang> {
-    core,
-    id: "unused-suppression".to_string(),
-    severity,
-    files: None,
-    ignores: None,
-    language: "rust".parse().unwrap(),
-    message: "Unused 'ast-grep-ignore' directive.".into(),
-    metadata: None,
-    note: None,
-    rewriters: None,
-    url: None,
-  };
-  RuleConfig::try_from(config, &Default::default()).unwrap()
+    .unwrap_or_else(|| default_unused_suppression_rule_severity(arg));
+  CombinedScan::unused_config(severity, SupportLang::Rust.into())
 }
 
-fn print_unused_suppressions(
-  path: &Path,
-  matches: Vec<NodeMatch<StrDoc<SgLang>>>,
-  rule_config: &RuleConfig<SgLang>,
-  file_content: &String,
-  printer: &impl Printer,
-) -> Result<usize> {
-  let count = match rule_config.severity {
-    Severity::Error => matches.len(),
-    // skip printing turned-off rule
-    Severity::Off => return Ok(0),
-    _ => 0,
-  };
-  match_rule_on_file(path, matches, rule_config, file_content, printer)?;
-  Ok(count)
-}
-
-impl<P: Printer> PathWorker for ScanWithConfig<P> {
+impl PathWorker for ScanWithConfig {
   fn get_trace(&self) -> &FileTrace {
     &self.trace.file_trace
   }
@@ -248,12 +212,11 @@ impl<P: Printer> PathWorker for ScanWithConfig<P> {
   }
 }
 
-struct ScanWithRule<Printer> {
-  printer: Printer,
+struct ScanWithRule {
   rules: Vec<RuleConfig<SgLang>>,
 }
-impl<P: Printer> ScanWithRule<P> {
-  fn try_new(arg: ScanArg, printer: P) -> Result<Self> {
+impl ScanWithRule {
+  fn try_new(arg: ScanArg) -> Result<Self> {
     let rules = if let Some(path) = &arg.rule {
       read_rule_file(path, None)?
     } else if let Some(text) = &arg.inline_rules {
@@ -262,29 +225,28 @@ impl<P: Printer> ScanWithRule<P> {
     } else {
       return Err(anyhow::anyhow!(EC::RuleNotSpecified));
     };
-    Ok(Self { printer, rules })
+    Ok(Self { rules })
   }
 }
 
-impl<P: Printer> Worker for ScanWithRule<P> {
+impl Worker for ScanWithRule {
   type Item = (PathBuf, AstGrep, PreScan);
-  fn consume_items(&self, items: Items<Self::Item>) -> Result<()> {
-    self.printer.before_print()?;
+  fn consume_items<P: Printer>(&self, items: Items<Self::Item>, mut printer: P) -> Result<()> {
+    printer.before_print()?;
     let mut error_count = 0usize;
     let combined = CombinedScan::new(self.rules.iter().collect());
     for (path, grep, pre_scan) in items {
       let file_content = grep.source().to_string();
       // do not exclude_fix rule in run_with_rule
       let scanned = combined.scan(&grep, pre_scan, false);
-      for (idx, matches) in scanned.matches {
-        let rule = combined.get_rule(idx);
+      for (rule, matches) in scanned.matches {
         if matches!(rule.severity, Severity::Error) {
           error_count = error_count.saturating_add(matches.len());
         }
-        match_rule_on_file(&path, matches, rule, &file_content, &self.printer)?;
+        match_rule_on_file(&path, matches, rule, &file_content, &mut printer)?;
       }
     }
-    self.printer.after_print()?;
+    printer.after_print()?;
     if error_count > 0 {
       Err(anyhow::anyhow!(EC::DiagnosticError(error_count)))
     } else {
@@ -293,7 +255,7 @@ impl<P: Printer> Worker for ScanWithRule<P> {
   }
 }
 
-impl<P: Printer> StdInWorker for ScanWithRule<P> {
+impl StdInWorker for ScanWithRule {
   fn parse_stdin(&self, src: String) -> Option<Self::Item> {
     use ast_grep_core::Language;
     let lang = self.rules[0].language;
@@ -309,12 +271,12 @@ impl<P: Printer> StdInWorker for ScanWithRule<P> {
 }
 fn match_rule_diff_on_file(
   path: &Path,
-  matches: Vec<(NodeMatch<StrDoc<SgLang>>, &RuleConfig<SgLang>)>,
-  reporter: &impl Printer,
+  matches: Vec<(&RuleConfig<SgLang>, NodeMatch<StrDoc<SgLang>>)>,
+  reporter: &mut impl Printer,
 ) -> Result<()> {
   let diffs = matches
     .into_iter()
-    .filter_map(|(m, rule)| {
+    .filter_map(|(rule, m)| {
       let fix = rule.matcher.fixer.as_ref()?;
       let diff = Diff::generate(m, &rule.matcher, fix);
       Some((diff, rule))
@@ -329,7 +291,7 @@ fn match_rule_on_file(
   matches: Vec<NodeMatch<StrDoc<SgLang>>>,
   rule: &RuleConfig<SgLang>,
   file_content: &String,
-  reporter: &impl Printer,
+  reporter: &mut impl Printer,
 ) -> Result<()> {
   let matches = matches.into_iter();
   let file = SimpleFile::new(path.to_string_lossy(), file_content);
