@@ -1,3 +1,6 @@
+use ast_grep_core::language::TSLanguage;
+use ast_grep_core::Language;
+use ast_grep_dynamic::{CustomLang, DynamicLang};
 use ast_grep_language::SupportLang;
 use ignore::types::{Types, TypesBuilder};
 use ignore::{WalkBuilder, WalkParallel};
@@ -6,7 +9,9 @@ use napi::anyhow::Error;
 use napi::bindgen_prelude::Result;
 use napi_derive::napi;
 
+use std::borrow::Cow;
 use std::collections::HashMap;
+use std::fmt::{Debug, Display, Formatter};
 use std::path::Path;
 use std::str::FromStr;
 
@@ -39,73 +44,27 @@ pub enum Lang {
   Yaml,
 }
 
-impl From<Lang> for SupportLang {
-  fn from(val: Lang) -> Self {
-    use Lang as F;
-    use SupportLang as S;
-    match val {
-      F::Html => S::Html,
-      F::JavaScript => S::JavaScript,
-      F::Tsx => S::Tsx,
-      F::Css => S::Css,
-      F::TypeScript => S::TypeScript,
-      F::Bash => S::Bash,
-      F::C => S::C,
-      F::Cpp => S::Cpp,
-      F::CSharp => S::CSharp,
-      F::Go => S::Go,
-      F::Elixir => S::Elixir,
-      F::Haskell => S::Haskell,
-      F::Java => S::Java,
-      F::Json => S::Json,
-      F::Kotlin => S::Kotlin,
-      F::Lua => S::Lua,
-      F::Php => S::Php,
-      F::Python => S::Python,
-      F::Ruby => S::Ruby,
-      F::Rust => S::Rust,
-      F::Scala => S::Scala,
-      F::Sql => S::Sql,
-      F::Swift => S::Swift,
-      F::Yaml => S::Yaml,
-    }
-  }
+#[derive(Copy, Clone, PartialEq, Eq, Hash)]
+pub enum NapiLang {
+  // inlined support lang expando char
+  Builtin(SupportLang),
+  Custom(DynamicLang),
 }
 
-impl From<SupportLang> for Lang {
-  fn from(value: SupportLang) -> Self {
-    use Lang as F;
-    use SupportLang as S;
-    match value {
-      S::Html => F::Html,
-      S::JavaScript => F::JavaScript,
-      S::Tsx => F::Tsx,
-      S::Css => F::Css,
-      S::TypeScript => F::TypeScript,
-      S::Bash => F::Bash,
-      S::C => F::C,
-      S::Cpp => F::Cpp,
-      S::CSharp => F::CSharp,
-      S::Go => F::Go,
-      S::Elixir => F::Elixir,
-      S::Haskell => F::Haskell,
-      S::Java => F::Java,
-      S::Json => F::Json,
-      S::Kotlin => F::Kotlin,
-      S::Lua => F::Lua,
-      S::Php => F::Php,
-      S::Python => F::Python,
-      S::Ruby => F::Ruby,
-      S::Rust => F::Rust,
-      S::Scala => F::Scala,
-      S::Sql => F::Sql,
-      S::Swift => F::Swift,
-      S::Yaml => F::Yaml,
+impl NapiLang {
+  fn all_langs() -> Vec<Self> {
+    let builtin = SupportLang::all_langs().iter().copied().map(Self::Builtin);
+    let customs = DynamicLang::all_langs().into_iter().map(Self::Custom);
+    builtin.chain(customs).collect()
+  }
+
+  fn file_types(&self) -> Types {
+    match self {
+      Builtin(b) => b.file_types(),
+      Custom(c) => c.file_types(),
     }
   }
-}
 
-impl Lang {
   pub fn find_files(
     &self,
     paths: Vec<String>,
@@ -116,7 +75,7 @@ impl Lang {
   pub fn lang_globs(map: HashMap<String, Vec<String>>) -> LanguageGlobs {
     let mut ret = HashMap::new();
     for (name, patterns) in map {
-      if let Ok(lang) = Lang::from_str(&name) {
+      if let Ok(lang) = NapiLang::from_str(&name) {
         ret.insert(lang, patterns);
       }
     }
@@ -124,31 +83,98 @@ impl Lang {
   }
 }
 
-pub type LanguageGlobs = HashMap<Lang, Vec<String>>;
-
-impl FromStr for Lang {
-  type Err = Error;
-  fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-    SupportLang::from_str(s)
-      .map(|l| l.into())
-      .map_err(|_| anyhow!(format!("{s} is not supported in napi")))
+impl Display for NapiLang {
+  fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
+    match self {
+      Builtin(b) => write!(f, "{}", b),
+      Custom(c) => write!(f, "{}", c.name()),
+    }
   }
 }
 
+impl Debug for NapiLang {
+  fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
+    match self {
+      Builtin(b) => write!(f, "{:?}", b),
+      Custom(c) => write!(f, "{:?}", c.name()),
+    }
+  }
+}
+
+impl FromStr for NapiLang {
+  type Err = Error;
+  fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
+    if let Ok(b) = SupportLang::from_str(s) {
+      Ok(NapiLang::Builtin(b))
+    } else if let Ok(c) = DynamicLang::from_str(s) {
+      Ok(NapiLang::Custom(c))
+    } else {
+      Err(anyhow!(format!("{s} is not supported in napi")))
+    }
+  }
+}
+
+impl From<SupportLang> for NapiLang {
+  fn from(val: SupportLang) -> Self {
+    NapiLang::Builtin(val)
+  }
+}
+
+use NapiLang::*;
+impl Language for NapiLang {
+  fn get_ts_language(&self) -> TSLanguage {
+    match self {
+      Builtin(b) => b.get_ts_language(),
+      Custom(c) => c.get_ts_language(),
+    }
+  }
+
+  fn pre_process_pattern<'q>(&self, query: &'q str) -> Cow<'q, str> {
+    match self {
+      Builtin(b) => b.pre_process_pattern(query),
+      Custom(c) => c.pre_process_pattern(query),
+    }
+  }
+
+  #[inline]
+  fn meta_var_char(&self) -> char {
+    match self {
+      Builtin(b) => b.meta_var_char(),
+      Custom(c) => c.meta_var_char(),
+    }
+  }
+
+  #[inline]
+  fn expando_char(&self) -> char {
+    match self {
+      Builtin(b) => b.expando_char(),
+      Custom(c) => c.expando_char(),
+    }
+  }
+}
+
+pub fn register_dynamic_language(langs: HashMap<String, CustomLang>) -> Result<()> {
+  let base = std::env::current_dir()?;
+  CustomLang::register(&base, langs).expect("TODO");
+  Ok(())
+}
+
+pub type LanguageGlobs = HashMap<NapiLang, Vec<String>>;
+
 pub enum LangOption {
   /// Used when language is inferred from file path
   /// e.g. in parse_files
-  Inferred(Vec<(SupportLang, Types)>),
+  Inferred(Vec<(NapiLang, Types)>),
   /// Used when language is specified
   /// e.g. in frontend_lang.find_in_files
-  Specified(Lang),
+  Specified(NapiLang),
 }
 
 impl LangOption {
-  pub fn get_lang(&self, path: &Path) -> Option<SupportLang> {
+  pub fn get_lang(&self, path: &Path) -> Option<NapiLang> {
     use LangOption::*;
     match self {
-      Specified(lang) => Some((*lang).into()),
+      Specified(lang) => Some(*lang),
       Inferred(pairs) => pairs
         .iter()
         .find_map(|(lang, types)| types.matched(path, false).is_whitelist().then_some(*lang)),
@@ -157,17 +183,16 @@ impl LangOption {
   pub fn infer(language_globs: &LanguageGlobs) -> Self {
     let mut types = vec![];
     let empty = vec![];
-    for lang in SupportLang::all_langs() {
+    for lang in NapiLang::all_langs() {
       let mut builder = TypesBuilder::new();
       let tpe = lang.to_string();
       let file_types = lang.file_types();
       add_types(&mut builder, &file_types);
-      let fe_lang = Lang::from(*lang);
-      for pattern in language_globs.get(&fe_lang).unwrap_or(&empty) {
+      for pattern in language_globs.get(&lang).unwrap_or(&empty) {
         builder.add(&tpe, pattern).expect("should build");
       }
       builder.select(&tpe);
-      types.push((*lang, builder.build().unwrap()));
+      types.push((lang, builder.build().unwrap()));
     }
     Self::Inferred(types)
   }
@@ -179,10 +204,9 @@ pub fn build_files(paths: Vec<String>, language_globs: &LanguageGlobs) -> Result
   }
   let mut types = TypesBuilder::new();
   let empty = vec![];
-  for lang in SupportLang::all_langs() {
+  for lang in NapiLang::all_langs() {
     let type_name = lang.to_string();
-    let l = Lang::from(*lang);
-    let custom = language_globs.get(&l).unwrap_or(&empty);
+    let custom = language_globs.get(&lang).unwrap_or(&empty);
     let default_types = lang.file_types();
     select_custom(&mut types, &type_name, &default_types, custom);
   }
@@ -221,7 +245,7 @@ fn select_custom<'b>(
 }
 
 fn find_files_with_lang(
-  lang: &Lang,
+  lang: &NapiLang,
   paths: Vec<String>,
   language_globs: Option<Vec<String>>,
 ) -> Result<WalkParallel> {
@@ -230,10 +254,9 @@ fn find_files_with_lang(
   }
 
   let mut types = TypesBuilder::new();
-  let sg_lang: SupportLang = (*lang).into();
-  let type_name = sg_lang.to_string();
+  let type_name = lang.to_string();
   let custom_file_type = language_globs.unwrap_or_default();
-  let default_types = sg_lang.file_types();
+  let default_types = lang.file_types();
   let types = select_custom(&mut types, &type_name, &default_types, &custom_file_type)
     .build()
     .unwrap();
@@ -250,18 +273,18 @@ fn find_files_with_lang(
 mod test {
   use super::*;
 
-  fn lang_globs() -> HashMap<Lang, Vec<String>> {
+  fn lang_globs() -> LanguageGlobs {
     let mut lang = HashMap::new();
     lang.insert("html".into(), vec!["*.vue".into()]);
-    Lang::lang_globs(lang)
+    NapiLang::lang_globs(lang)
   }
 
   #[test]
   fn test_lang_globs() {
     let globs = lang_globs();
-    assert!(globs.contains_key(&Lang::Html));
-    assert!(!globs.contains_key(&Lang::Tsx));
-    assert_eq!(globs[&Lang::Html], vec!["*.vue"]);
+    assert!(globs.contains_key(&SupportLang::Html.into()));
+    assert!(!globs.contains_key(&SupportLang::Tsx.into()));
+    assert_eq!(globs[&NapiLang::Builtin(SupportLang::Html)], vec!["*.vue"]);
   }
 
   #[test]
@@ -269,24 +292,24 @@ mod test {
     let globs = lang_globs();
     let option = LangOption::infer(&globs);
     let lang = option.get_lang(Path::new("test.vue"));
-    assert_eq!(lang, Some(SupportLang::Html));
+    assert_eq!(lang, Some(SupportLang::Html.into()));
     let lang = option.get_lang(Path::new("test.html"));
-    assert_eq!(lang, Some(SupportLang::Html));
+    assert_eq!(lang, Some(SupportLang::Html.into()));
     let lang = option.get_lang(Path::new("test.js"));
-    assert_eq!(lang, Some(SupportLang::JavaScript));
+    assert_eq!(lang, Some(SupportLang::JavaScript.into()));
     let lang = option.get_lang(Path::new("test.xss"));
     assert_eq!(lang, None);
   }
 
   #[test]
   fn test_from_str() {
-    let lang = Lang::from_str("html");
-    assert_eq!(lang.unwrap(), Lang::Html);
-    let lang = Lang::from_str("Html");
-    assert_eq!(lang.unwrap(), Lang::Html);
-    let lang = Lang::from_str("htML");
-    assert_eq!(lang.unwrap(), Lang::Html);
-    let lang = Lang::from_str("ocaml");
+    let lang = NapiLang::from_str("html");
+    assert_eq!(lang.unwrap(), SupportLang::Html.into());
+    let lang = NapiLang::from_str("Html");
+    assert_eq!(lang.unwrap(), SupportLang::Html.into());
+    let lang = NapiLang::from_str("htML");
+    assert_eq!(lang.unwrap(), SupportLang::Html.into());
+    let lang = NapiLang::from_str("ocaml");
     assert!(lang.is_err());
   }
 }
